@@ -4,27 +4,30 @@ import (
 	"context"
 	"strings"
 
+	liberr "github.com/konveyor/controller/pkg/error"
 	migapi "github.com/konveyor/mig-controller/pkg/apis/migration/v1alpha1"
-	"github.com/konveyor/mig-controller/pkg/gvk"
+	"github.com/konveyor/mig-controller/pkg/compat"
+	imagev1 "github.com/openshift/api/image/v1"
 	corev1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/errors"
-	k8serror "k8s.io/apimachinery/pkg/api/errors"
-	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	k8sclient "sigs.k8s.io/controller-runtime/pkg/client"
 )
 
 // Velero Plugin Annotations
 const (
-	StageOrFinalAnnotation   = "openshift.io/migrate-copy-phase"   // (stage|final)
-	PvActionAnnotation       = "openshift.io/migrate-type"         // (move|copy)
-	PvStorageClassAnnotation = "openshift.io/target-storage-class" // storageClassName
-	PvAccessModeAnnotation   = "openshift.io/target-access-mode"   // accessMode
-	QuiesceAnnotation        = "openshift.io/migrate-quiesce-pods" // (true|false)
-	QuiesceNodeSelector      = "migration.openshift.io/quiesceDaemonSet"
-	SuspendAnnotation        = "migration.openshift.io/preQuiesceSuspend"
-	ReplicasAnnotation       = "migration.openshift.io/preQuiesceReplicas"
-	NodeSelectorAnnotation   = "migration.openshift.io/preQuiesceNodeSelector"
-	StagePodImageAnnotation  = "migration.openshift.io/stage-pod-image"
+	StageOrFinalMigrationAnnotation = "migration.openshift.io/migmigration-type" // (stage|final)
+	StageMigration                  = "stage"
+	FinalMigration                  = "final"
+	PvActionAnnotation              = "openshift.io/migrate-type"          // (move|copy)
+	PvStorageClassAnnotation        = "openshift.io/target-storage-class"  // storageClassName
+	PvAccessModeAnnotation          = "openshift.io/target-access-mode"    // accessMode
+	PvCopyMethodAnnotation          = "migration.openshift.io/copy-method" // (snapshot|filesystem)
+	QuiesceAnnotation               = "openshift.io/migrate-quiesce-pods"  // (true|false)
+	QuiesceNodeSelector             = "migration.openshift.io/quiesceDaemonSet"
+	SuspendAnnotation               = "migration.openshift.io/preQuiesceSuspend"
+	ReplicasAnnotation              = "migration.openshift.io/preQuiesceReplicas"
+	NodeSelectorAnnotation          = "migration.openshift.io/preQuiesceNodeSelector"
+	StagePodImageAnnotation         = "migration.openshift.io/stage-pod-image"
 )
 
 // Restic Annotations
@@ -54,6 +57,14 @@ const (
 	// for easy search or application rollback.
 	// The value is the Task.UID().
 	MigratedByLabel = "migration.openshift.io/migrated-by" // (migmigration UID)
+	// Identifies associated migmigration
+	// to assist manual debugging
+	// The value is Task.Owner.Name
+	MigMigrationDebugLabel = "migration.openshift.io/migmigration-name"
+	// Identifies associated migplan
+	// to assist manual debugging
+	// The value is Task.Owner.Spec.migPlanRef.Name
+	MigPlanDebugLabel = "migration.openshift.io/migplan-name"
 )
 
 // Set of Service Accounts.
@@ -64,41 +75,42 @@ type ServiceAccounts map[string]map[string]bool
 // The PvActionAnnotation annotation is added to PV & PVC as needed by the velero plugin.
 // The PvStorageClassAnnotation annotation is added to PVC as needed by the velero plugin.
 // The PvAccessModeAnnotation annotation is added to PVC as needed by the velero plugin.
+// The PvCopyMethodAnnotation annotation is added to PVC as needed by the velero plugin.
 // The ResticPvBackupAnnotation is added to Pods as needed by Restic.
 // The ResticPvVerifyAnnotation is added to Pods as needed by Restic.
-// The IncludedInStageBackupLabel label is added to Pods, PVs, PVCs and is referenced
-// by the velero.Backup label selector.
+// The IncludedInStageBackupLabel label is added to Pods, PVs, PVCs, ImageStreams and
+// is referenced by the velero.Backup label selector.
 // The IncludedInStageBackupLabel label is added to Namespaces to prevent the
 // velero.Backup from being empty which causes the Restore to fail.
 func (t *Task) annotateStageResources() error {
-	client, err := t.getSourceClient()
+	sourceClient, err := t.getSourceClient()
 	if err != nil {
-		log.Trace(err)
-		return err
+		return liberr.Wrap(err)
 	}
 	// Namespaces
-	err = t.labelNamespaces(client)
+	err = t.labelNamespaces(sourceClient)
 	if err != nil {
-		log.Trace(err)
-		return err
+		return liberr.Wrap(err)
 	}
 	// Pods
-	serviceAccounts, err := t.annotatePods(client)
+	serviceAccounts, err := t.annotatePods(sourceClient)
 	if err != nil {
-		log.Trace(err)
-		return err
+		return liberr.Wrap(err)
 	}
 	// PV & PVCs
-	err = t.annotatePVs(client)
+	err = t.annotatePVs(sourceClient)
 	if err != nil {
-		log.Trace(err)
-		return err
+		return liberr.Wrap(err)
 	}
 	// Service accounts used by stage pods.
-	err = t.labelServiceAccounts(client, serviceAccounts)
+	err = t.labelServiceAccounts(sourceClient, serviceAccounts)
 	if err != nil {
-		log.Trace(err)
-		return err
+		return liberr.Wrap(err)
+	}
+
+	err = t.labelImageStreams(sourceClient)
+	if err != nil {
+		return liberr.Wrap(err)
 	}
 
 	return nil
@@ -123,8 +135,7 @@ func (t *Task) getResticVolumes(client k8sclient.Client, pod corev1.Pod) ([]stri
 			},
 			&pvc)
 		if err != nil {
-			log.Trace(err)
-			return nil, nil, err
+			return nil, nil, liberr.Wrap(err)
 		}
 		pvAction := findPVAction(pvs, pvc.Spec.VolumeName)
 		if pvAction == migapi.PvCopyAction {
@@ -151,8 +162,7 @@ func (t *Task) labelNamespaces(client k8sclient.Client) error {
 			},
 			&namespace)
 		if err != nil {
-			log.Trace(err)
-			return err
+			return liberr.Wrap(err)
 		}
 		if namespace.Labels == nil {
 			namespace.Labels = make(map[string]string)
@@ -160,8 +170,7 @@ func (t *Task) labelNamespaces(client k8sclient.Client) error {
 		namespace.Labels[IncludedInStageBackupLabel] = t.UID()
 		err = client.Update(context.TODO(), &namespace)
 		if err != nil {
-			log.Trace(err)
-			return err
+			return liberr.Wrap(err)
 		}
 		log.Info(
 			"NS annotations/labels added.",
@@ -184,15 +193,13 @@ func (t *Task) annotatePods(client k8sclient.Client) (ServiceAccounts, error) {
 	options := k8sclient.MatchingLabels(t.Owner.GetCorrelationLabels())
 	err := client.List(context.TODO(), options, &list)
 	if err != nil {
-		log.Trace(err)
-		return nil, err
+		return nil, liberr.Wrap(err)
 	}
 	for _, pod := range list.Items {
 		// Annotate PVCs.
 		volumes, verifyVolumes, err := t.getResticVolumes(client, pod)
 		if err != nil {
-			log.Trace(err)
-			return nil, err
+			return nil, liberr.Wrap(err)
 		}
 		// Restic annotation used to specify volumes.
 		if pod.Annotations == nil {
@@ -203,8 +210,7 @@ func (t *Task) annotatePods(client k8sclient.Client) (ServiceAccounts, error) {
 		// Update
 		err = client.Update(context.TODO(), &pod)
 		if err != nil {
-			log.Trace(err)
-			return nil, err
+			return nil, liberr.Wrap(err)
 		}
 		log.Info(
 			"Pod annotations/labels added.",
@@ -230,6 +236,7 @@ func (t *Task) annotatePods(client k8sclient.Client) (ServiceAccounts, error) {
 // The IncludedInStageBackupLabel label is added to PVs and PVCs and is referenced
 // by the velero.Backup label selector.
 // The PvAccessModeAnnotation annotation is added to PVC as needed by the velero plugin.
+// The PvCopyMethodAnnotation annotation is added to PV and PVC as needed by the velero plugin.
 func (t *Task) annotatePVs(client k8sclient.Client) error {
 	pvs := t.getPVs()
 	for _, pv := range pvs.List {
@@ -241,8 +248,7 @@ func (t *Task) annotatePVs(client k8sclient.Client) error {
 			},
 			&pvResource)
 		if err != nil {
-			log.Trace(err)
-			return err
+			return liberr.Wrap(err)
 		}
 		if pvResource.Annotations == nil {
 			pvResource.Annotations = make(map[string]string)
@@ -251,6 +257,10 @@ func (t *Task) annotatePVs(client k8sclient.Client) error {
 		pvResource.Annotations[PvActionAnnotation] = pv.Selection.Action
 		// PV storageClass annotation needed by the velero plugin.
 		pvResource.Annotations[PvStorageClassAnnotation] = pv.Selection.StorageClass
+		if pv.Selection.Action == migapi.PvCopyAction {
+			// PV copyMethod annotation needed by the velero plugin.
+			pvResource.Annotations[PvCopyMethodAnnotation] = pv.Selection.CopyMethod
+		}
 		// Add label used by stage backup label selector.
 		if pvResource.Labels == nil {
 			pvResource.Labels = make(map[string]string)
@@ -259,8 +269,7 @@ func (t *Task) annotatePVs(client k8sclient.Client) error {
 		// Update
 		err = client.Update(context.TODO(), &pvResource)
 		if err != nil {
-			log.Trace(err)
-			return err
+			return liberr.Wrap(err)
 		}
 		log.Info(
 			"PV annotations/labels added.",
@@ -276,8 +285,7 @@ func (t *Task) annotatePVs(client k8sclient.Client) error {
 			},
 			&pvcResource)
 		if err != nil {
-			log.Trace(err)
-			return err
+			return liberr.Wrap(err)
 		}
 		if pvcResource.Annotations == nil {
 			pvcResource.Annotations = make(map[string]string)
@@ -292,6 +300,8 @@ func (t *Task) annotatePVs(client k8sclient.Client) error {
 		if pv.Selection.Action == migapi.PvCopyAction {
 			// PV storageClass annotation needed by the velero plugin.
 			pvcResource.Annotations[PvStorageClassAnnotation] = pv.Selection.StorageClass
+			// PV copyMethod annotation needed by the velero plugin.
+			pvcResource.Annotations[PvCopyMethodAnnotation] = pv.Selection.CopyMethod
 			// PV accessMode annotation needed by the velero plugin, if present on the PV.
 			if pv.Selection.AccessMode != "" {
 				pvcResource.Annotations[PvAccessModeAnnotation] = string(pv.Selection.AccessMode)
@@ -300,8 +310,7 @@ func (t *Task) annotatePVs(client k8sclient.Client) error {
 		// Update
 		err = client.Update(context.TODO(), &pvcResource)
 		if err != nil {
-			log.Trace(err)
-			return err
+			return liberr.Wrap(err)
 		}
 		log.Info(
 			"PVC annotations/labels added.",
@@ -325,8 +334,7 @@ func (t *Task) labelServiceAccounts(client k8sclient.Client, serviceAccounts Ser
 		options := k8sclient.InNamespace(ns)
 		err := client.List(context.TODO(), options, &list)
 		if err != nil {
-			log.Trace(err)
-			return err
+			return liberr.Wrap(err)
 		}
 		for _, sa := range list.Items {
 			if _, found := names[sa.Name]; !found {
@@ -338,8 +346,7 @@ func (t *Task) labelServiceAccounts(client k8sclient.Client, serviceAccounts Ser
 			sa.Labels[IncludedInStageBackupLabel] = t.UID()
 			err = client.Update(context.TODO(), &sa)
 			if err != nil {
-				log.Trace(err)
-				return err
+				return liberr.Wrap(err)
 			}
 			log.Info(
 				"SA annotations/labels added.",
@@ -353,39 +360,68 @@ func (t *Task) labelServiceAccounts(client k8sclient.Client, serviceAccounts Ser
 	return nil
 }
 
+// Add label to ImageStreeams
+func (t *Task) labelImageStreams(client compat.Client) error {
+	for _, ns := range t.sourceNamespaces() {
+		imageStreamList := imagev1.ImageStreamList{}
+		options := k8sclient.InNamespace(ns)
+		err := client.List(context.TODO(), options, &imageStreamList)
+		if err != nil {
+			log.Trace(err)
+			return err
+		}
+		for _, is := range imageStreamList.Items {
+			if is.Labels == nil {
+				is.Labels = map[string]string{}
+			}
+			is.Labels[IncludedInStageBackupLabel] = t.UID()
+			err = client.Update(context.Background(), &is)
+			if err != nil {
+				return liberr.Wrap(err)
+			}
+			log.Info(
+				"ImageStream labels added.",
+				"ns",
+				is.Namespace,
+				"name",
+				is.Name)
+		}
+	}
+
+	return nil
+}
+
 // Delete temporary annotations and labels added.
 func (t *Task) deleteAnnotations() error {
 	clients, namespaceList, err := t.getBothClientsWithNamespaces()
 	if err != nil {
-		log.Trace(err)
-		return err
+		return liberr.Wrap(err)
 	}
 
 	for i, client := range clients {
 		err = t.deletePVCAnnotations(client, namespaceList[i])
 		if err != nil {
-			log.Trace(err)
-			return err
+			return liberr.Wrap(err)
 		}
 		err = t.deletePVAnnotations(client)
 		if err != nil {
-			log.Trace(err)
-			return err
+			return liberr.Wrap(err)
 		}
 		err = t.deletePodAnnotations(client, namespaceList[i])
 		if err != nil {
-			log.Trace(err)
-			return err
+			return liberr.Wrap(err)
 		}
 		err = t.deleteNamespaceLabels(client, namespaceList[i])
 		if err != nil {
-			log.Trace(err)
-			return err
+			return liberr.Wrap(err)
 		}
 		err = t.deleteServiceAccountLabels(client)
 		if err != nil {
-			log.Trace(err)
-			return err
+			return liberr.Wrap(err)
+		}
+		err = t.deleteImageStreamLabels(client, namespaceList[i])
+		if err != nil {
+			return liberr.Wrap(err)
 		}
 	}
 
@@ -399,8 +435,7 @@ func (t *Task) deletePodAnnotations(client k8sclient.Client, namespaceList []str
 		podList := corev1.PodList{}
 		err := client.List(context.TODO(), options, &podList)
 		if err != nil {
-			log.Trace(err)
-			return err
+			return liberr.Wrap(err)
 		}
 		for _, pod := range podList.Items {
 			if !pod.ObjectMeta.DeletionTimestamp.IsZero() {
@@ -428,8 +463,7 @@ func (t *Task) deletePodAnnotations(client k8sclient.Client, namespaceList []str
 			}
 			err = client.Update(context.TODO(), &pod)
 			if err != nil {
-				log.Trace(err)
-				return err
+				return liberr.Wrap(err)
 			}
 			log.Info(
 				"Pod annotations/labels removed.",
@@ -459,60 +493,17 @@ func (t *Task) deleteNamespaceLabels(client k8sclient.Client, namespaceList []st
 			continue
 		}
 		if err != nil {
-			log.Trace(err)
-			return err
+			return liberr.Wrap(err)
 		}
 		delete(namespace.Labels, IncludedInStageBackupLabel)
 		err = client.Update(context.TODO(), &namespace)
 		if err != nil {
-			log.Trace(err)
-			return err
+			return liberr.Wrap(err)
 		}
 	}
 	return nil
 }
 
-// deleteLabels will delete all migration.openshift.io/migrated-by labels
-// from the application upon successful completion
-func (t *Task) deleteLabels() error {
-	client, GVRs, err := gvk.GetGVRsForCluster(t.PlanResources.DestMigCluster, t.Client)
-	if err != nil {
-		log.Trace(err)
-		return err
-	}
-
-	listOptions := k8sclient.MatchingLabels(map[string]string{
-		MigratedByLabel: string(t.Owner.UID),
-	}).AsListOptions()
-
-	for _, gvr := range GVRs {
-		for _, ns := range t.destinationNamespaces() {
-			list, err := client.Resource(gvr).Namespace(ns).List(*listOptions)
-			if err != nil {
-				log.Trace(err)
-				return err
-			}
-			for _, r := range list.Items {
-				labels := r.GetLabels()
-				delete(labels, MigratedByLabel)
-				r.SetLabels(labels)
-				_, err = client.Resource(gvr).Namespace(ns).Update(&r, metav1.UpdateOptions{})
-				if err != nil {
-					// The part touches the application on the destination side, fail safe to ensure
-					// this won't block the migration
-					if k8serror.IsMethodNotSupported(err) {
-						continue
-					}
-					log.Trace(err)
-					return err
-				}
-			}
-		}
-	}
-
-	return nil
-}
-
 // Delete PVC stage annotations and labels.
 func (t *Task) deletePVCAnnotations(client k8sclient.Client, namespaceList []string) error {
 	for _, ns := range namespaceList {
@@ -520,8 +511,7 @@ func (t *Task) deletePVCAnnotations(client k8sclient.Client, namespaceList []str
 		pvcList := corev1.PersistentVolumeClaimList{}
 		err := client.List(context.TODO(), options, &pvcList)
 		if err != nil {
-			log.Trace(err)
-			return err
+			return liberr.Wrap(err)
 		}
 		for _, pvc := range pvcList.Items {
 			if pvc.Spec.VolumeName == "" {
@@ -553,8 +543,7 @@ func (t *Task) deletePVCAnnotations(client k8sclient.Client, namespaceList []str
 			}
 			err = client.Update(context.TODO(), &pvc)
 			if err != nil {
-				log.Trace(err)
-				return err
+				return liberr.Wrap(err)
 			}
 			log.Info(
 				"PVC annotations/labels removed.",
@@ -577,8 +566,7 @@ func (t *Task) deletePVAnnotations(client k8sclient.Client) error {
 	pvList := corev1.PersistentVolumeList{}
 	err := client.List(context.TODO(), options, &pvList)
 	if err != nil {
-		log.Trace(err)
-		return err
+		return liberr.Wrap(err)
 	}
 	for _, pv := range pvList.Items {
 		delete(pv.Labels, IncludedInStageBackupLabel)
@@ -586,8 +574,7 @@ func (t *Task) deletePVAnnotations(client k8sclient.Client) error {
 		delete(pv.Annotations, PvStorageClassAnnotation)
 		err = client.Update(context.TODO(), &pv)
 		if err != nil {
-			log.Trace(err)
-			return err
+			return liberr.Wrap(err)
 		}
 		log.Info(
 			"PV annotations/labels removed.",
@@ -607,15 +594,13 @@ func (t *Task) deleteServiceAccountLabels(client k8sclient.Client) error {
 	pvList := corev1.ServiceAccountList{}
 	err := client.List(context.TODO(), options, &pvList)
 	if err != nil {
-		log.Trace(err)
-		return err
+		return liberr.Wrap(err)
 	}
 	for _, sa := range pvList.Items {
 		delete(sa.Labels, IncludedInStageBackupLabel)
 		err = client.Update(context.TODO(), &sa)
 		if err != nil {
-			log.Trace(err)
-			return err
+			return liberr.Wrap(err)
 		}
 		log.Info(
 			"SA annotations/labels removed.",
@@ -627,3 +612,30 @@ func (t *Task) deleteServiceAccountLabels(client k8sclient.Client) error {
 
 	return nil
 }
+
+// Delete ImageStream labels
+func (t *Task) deleteImageStreamLabels(client k8sclient.Client, namespaceList []string) error {
+	for _, ns := range namespaceList {
+		imageStreamList := imagev1.ImageStreamList{}
+		options := k8sclient.InNamespace(ns)
+		err := client.List(context.TODO(), options, &imageStreamList)
+		if err != nil {
+			log.Trace(err)
+			return err
+		}
+		for _, is := range imageStreamList.Items {
+			delete(is.Labels, IncludedInStageBackupLabel)
+			err = client.Update(context.Background(), &is)
+			if err != nil {
+				return liberr.Wrap(err)
+			}
+			log.Info(
+				"ImageStream labels removed.",
+				"ns",
+				is.Namespace,
+				"name",
+				is.Name)
+		}
+	}
+	return nil
+}
